

% Estonia MPA vs. wind farm study - last update 2016-01-07
% The script uses DCE estimation package available at czaj.org (CC BY 4.0) 

clear all
clc

global B_backup; % this is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading & specifying data ****************************


DATA = load('data_EE1_01');

DATA.SQ = DATA.alt == 1;
DATA.AS_MPA = DATA.as == 1;
DATA.AS_WP = DATA.as == 2;
DATA.AS_EWP = DATA.as == 3;
DATA.WS_MPA = DATA.ws == 1;
DATA.WS_WP = DATA.ws == 2;
DATA.WS_EWP = DATA.ws == 3;

DATA.X = [DATA.SQ, DATA.AS_MPA, DATA.AS_WP, DATA.AS_EWP, DATA.WS_MPA, DATA.WS_WP, DATA.WS_EWP, -DATA.c/10];
DATA.Y = DATA.ans1;

EstimOpt.NamesA = {'SQ';'AS_MPA';'AS_WP';'AS_EWP';'WS_MPA';'WS_WP';'WS_EWP';'COST'};


%% ****************************  loading & specifying input ****************************


INPUT.Y = DATA.Y;
INPUT.Xa = DATA.X;


%% ****************************  sample characteristics ****************************


EstimOpt.NP = 800; % Number of respondents
EstimOpt.NCT = 12; % Number of choice tasks per person 
EstimOpt.NAlt = 3; % Number of alternatives


%% **************************** estimation and optimization options ****************************


% EstimOpt.eps = 1.e-9; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataClean(INPUT,EstimOpt);

EstimOpt.NRep = 1e4; % number of draws for numerical simulation
% OptimOpt.MaxIter = 1e3; % maximum number of iterations
% OptimOpt.Algorithm = 'quasi-newton'; % 'trust-region'
% EstimOpt.NumGrad = 1; % 1
% OptimOpt.GradObj = 'on'; % 'off'
% OptimOpt.FinDiffType = 'central'; % 'forward'
% OptimOpt.Hessian = 'user-supplied'; % 'off'
EstimOpt.HessEstFix = 1; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian


%% **************************** ModelOpt = MNL ****************************

% EstimOpt.WTP_space = 1;
 
% Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);

% xlswrite('MNL.xls', Results.MNL.R_out)


%% *************************** ModelOpt = LC ********************************

% EstimOpt.NClass = 3;

% Results.LC = LC(INPUT,Results,EstimOpt,OptimOpt);

% xlswrite('LC.xls', Results.LC.R_out) 

%% *************************** ModelOpt = LCMXL ********************************

EstimOpt.WTP_space = 1;

EstimOpt.NClass = 3;

EstimOpt.Scores = 1; % Predict individual-specific paramters / class membership probabilities using bayes rule

% LL: -5865.3321
Results.LCMXL_d.b0 = [1.12626744655728;0.0349124651245641;0.0492401174103733;-0.00654242237073613;0.0724285390622388;0.00300306404635841;-0.0213335002329972;5.75900463697419;-4.56580045019110;2.98788127787234;1.12910025692714;1.39353341390255;3.28329505631079;2.61941821625990;1.24164773102982;-1.37702569949525;-0.665382041241564;-1.77878683921978;-7.30009418271102;0.713114849236080;-0.326312273957866;-3.91024853253550;-0.0808181758663853;-0.0807426863402403;1.10927188935126;0.275101388600607;0.0906939766756917;0.430428540022181;0.0614455775784114;0.509722558314662;0.00113050521775534;2.04833891436366;5.19697818295998;0.700374494206406;-1.05619412710517;-0.364833914898256;-0.0652988676915059;-0.414206044940093;-0.803991465732790;0.914869144888161;1.45525504996039;1.92639160340442;1.71749037048098;1.43842601608745;1.81618021990200;2.02878266634747;1.38112823195723;0.792043684709823;-0.0582023271069065;0.0171381180594113];

Results.LCMXL_d = LCMXL(INPUT,Results,EstimOpt,OptimOpt);

xlswrite('LCMXL.xls', Results.LCMXL_d.R_out) 

%% welfare changes:

sim1 = 1e7;

B_i = mvnrnd(Results.LCMXL_d.bhat,Results.LCMXL_d.ihess,sim1);

WTP = 10 * ...
      [(-B_i(:,1)+B_i(:,2)+B_i(:,5)).*Results.LCMXL_d.PClass(1) + (-B_i(:,1+8)+B_i(:,2+8)+B_i(:,5+8)).*Results.LCMXL_d.PClass(2) + (-B_i(:,1+16)+B_i(:,2+16)+B_i(:,5+16)).*Results.LCMXL_d.PClass(3),...
       (-B_i(:,1)+B_i(:,3)+B_i(:,6)).*Results.LCMXL_d.PClass(1) + (-B_i(:,1+8)+B_i(:,3+8)+B_i(:,6+8)).*Results.LCMXL_d.PClass(2) + (-B_i(:,1+16)+B_i(:,3+16)+B_i(:,6+16)).*Results.LCMXL_d.PClass(3),...
       (-B_i(:,1)+B_i(:,4)+B_i(:,7)).*Results.LCMXL_d.PClass(1) + (-B_i(:,1+8)+B_i(:,4+8)+B_i(:,7+8)).*Results.LCMXL_d.PClass(2) + (-B_i(:,1+16)+B_i(:,4+16)+B_i(:,7+16)).*Results.LCMXL_d.PClass(3)];

WTPstats = [median(WTP);std(WTP);quantile(WTP,0.025);quantile(WTP,0.975)]


%% plots:

Results.LCMXL_d.BScores = Results.LCMXL_d.BScores * 10;

set(gcf,'defaultlinelinewidth',2);
figure(1)
[f,xi] = ksdensity(Results.LCMXL_d.BScores(:,1));
plot(xi,f,'LineWidth',2)
set(gcf,'Color', 'w')
title('Maintaing the status quo')
xlabel('WTP (EUR per year per household)')
ylabel('Density')
set(gca,'YTick', [])
set(gca,'Position',[.05 .12 .92 .82]);
% xlim([-20 20])

set(gcf,'defaultlinelinewidth',1);
figure(2)
[f,xi] = ksdensity(Results.LCMXL_d.BScores(:,2));
plot(xi,f,'LineWidth',2)
% set(gcf,'defaultlinelinewidth',2);
set(gcf,'Color', 'w')
title('Marine Protected Area on Apollo Shoal')
xlabel('WTP (EUR per year per household)')
ylabel('Density')
set(gca,'YTick', [])
set(gca,'Position',[.05 .12 .92 .82]);
% xlim([-2 4])

figure(3)
[f,xi] = ksdensity(Results.LCMXL_d.BScores(:,3));
plot(xi,f,'LineWidth',2)
% set(gcf,'defaultlinelinewidth',2);
set(gcf,'Color', 'w')
title('Wind Farm on Apollo Shoal')
xlabel('WTP (EUR per year per household)')
ylabel('Density')
set(gca,'YTick', [])
set(gca,'Position',[.05 .12 .92 .82]);
% xlim([-4 2])

figure(4)
[f,xi] = ksdensity(Results.LCMXL_d.BScores(:,4));
plot(xi,f,'LineWidth',2)
% set(gcf,'defaultlinelinewidth',2);
set(gcf,'Color', 'w')
title('Eco Wind Farm on Apollo Shoal')
xlabel('WTP (EUR per year per household)')
ylabel('Density')
set(gca,'YTick', [])
set(gca,'Position',[.05 .12 .92 .82]);
% xlim([-1 2])

figure(5)
[f,xi] = ksdensity(Results.LCMXL_d.BScores(:,5));
plot(xi,f,'LineWidth',2)
% set(gcf,'defaultlinelinewidth',2);
set(gcf,'Color', 'w')
title('Marine Protected Area on Western Shoals')
xlabel('WTP (EUR per year per household)')
ylabel('Density')
set(gca,'YTick', [])
set(gca,'Position',[.05 .12 .92 .82]);
% xlim([-1 3])

figure(6)
[f,xi] = ksdensity(Results.LCMXL_d.BScores(:,6));
plot(xi,f,'LineWidth',2)
% set(gcf,'defaultlinelinewidth',2);
set(gcf,'Color', 'w')
title('Wind Farm on Western Shoals')
xlabel('WTP (EUR per year per household)')
ylabel('Density')
set(gca,'YTick', [])
set(gca,'Position',[.05 .12 .92 .82]);
% xlim([-3 3])

figure(7)
[f,xi] = ksdensity(Results.LCMXL_d.BScores(:,7));
plot(xi,f,'LineWidth',2)
% set(gcf,'defaultlinelinewidth',2);
set(gcf,'Color', 'w')
title('Eco Wind Farm on Western Shoals')
xlabel('WTP (EUR per year per household)')
ylabel('Density')
set(gca,'YTick', [])
set(gca,'Position',[.05 .12 .92 .82]);
% xlim([-1 2])

% save EE1_final_out
